==> unixODBC-1.8.4.tar.gz: unixODBC-1.8.4/sqi/sqiColumns.c <==
#include "sqi.h"

int sqiColumns( HSQISTMT hStmt, char *pszTableName )
{
	HSQI		hSQI;
	HLST		hColHdrs;
	char		szBuffer[101];
	char		**pData;
	HSQICOLUMN	pColumn;
    FILE    	*hFile;
	char		szTableFile[FILENAME_MAX+1];

	if ( !hStmt )
		return false;
	
	if ( !pszTableName )
		return false;

	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	hSQI	= (HSQI)hStmt->hSQI;

	if ( !_sqiGetTableFileName( hStmt, hSQI->pszCatalogFile, pszTableName, szTableFile ) )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, "Could not get table file name" );
		return false;
	}

	hFile = fopen( szTableFile, "r" );
	if ( !hFile )
		hFile = fopen( szTableFile, "w" );

	if ( !hFile )
	{
		sprintf( hStmt->szMsg, "Could not open table file (%s)", szTableFile );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}

	hColHdrs = lstOpen();
	lstSetFreeFunc( hColHdrs, _sqiFreeColumnHeader );
	
	if ( !_sqiLoadTableColHdrs( hSQI->pszCatalogFile, pszTableName, hColHdrs, hFile ) )
	{
		sprintf( hStmt->szMsg, "Error while loading column headers from (%s)", szTableFile );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		lstClose( hColHdrs );
		fclose( hFile );
		return false;
	}

	fclose( hFile );

	if ( hColHdrs->nItems < 1 )
	{
		sprintf( hStmt->szMsg, "No column headers in (%s)", szTableFile );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		lstClose( hColHdrs );
		return false;
	}


	hStmt->hTableResult 				= malloc( sizeof(SQITABLE) );

	hStmt->hTableResult->pszTableName	= (char*)strdup( "sqiColumns" );
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->nRefs			= 0;
	hStmt->hTableResult->hBase			= NULL;

	/* table header */
	hStmt->hTableResult->hColHdrs	= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hColHdrs, _sqiFreeColumnHeader );

	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 0, "system", "sqiColumns", "TABLE_CAT" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 1, "system", "sqiColumns", "TABLE_SCHEM" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 2, "system", "sqiColumns", "TABLE_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 3, "system", "sqiColumns", "COLUMN_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 4, "system", "sqiColumns", "DATA_TYPE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 5, "system", "sqiColumns", "TYPE_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 6, "system", "sqiColumns", "COLUMN_SIZE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 7, "system", "sqiColumns", "BUFFER_LENGTH" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 8, "system", "sqiColumns", "NUM_PREC_RADIX" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 9, "system", "sqiColumns", "DECIMAL_DIGITS" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 10, "system", "sqiColumns", "NULLABLE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 11, "system", "sqiColumns", "REMARKS" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 12, "system", "sqiColumns", "COLUMN_DEF" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 13, "system", "sqiColumns", "SQL_DATA_TYPE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 14, "system", "sqiColumns", "SQL_DATETIME_SUB" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 15, "system", "sqiColumns", "CHAR_OCTET_LENGTH" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 16, "system", "sqiColumns", "ORDINAL_POSITION" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 17, "system", "sqiColumns", "IS_NULLABLE" ) );

	/* table rows (based upon root table col hdrs) */
	hStmt->hTableResult->hRows		= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hRows, _sqiFreeDataRow );

	lstFirst( hColHdrs );
	while ( !lstEOL( hColHdrs ) )
	{
		pData		= malloc( sizeof(char*) * 19 );
		pColumn 	= (HSQICOLUMN)lstGet( hColHdrs );
		pData[0]	= (char *)strdup( pColumn->pszSQL_DESC_CATALOG_NAME );
		pData[1]	= (char *)strdup( pColumn->pszSQL_DESC_SCHEMA_NAME );
		pData[2]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_TABLE_NAME );
		pData[3]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_COLUMN_NAME );
		pData[4]	= (char *)strdup( "" );
		pData[5]	= (char *)strdup( pColumn->pszSQL_DESC_TYPE_NAME );
		sprintf( szBuffer, "%d", pColumn->nSQL_DESC_LENGTH );
		pData[6]	= (char *)strdup( szBuffer );
		pData[7]	= (char *)strdup( "" );
		pData[8]	= (char *)strdup( "" );
		pData[9]	= (char *)strdup( "" );
		pData[10]	= (char *)strdup( (pColumn->nSQL_DESC_NULLABLE ? "0" : "1") );
		pData[11]	= (char *)strdup( "" );
		pData[12]	= (char *)strdup( "" );
		pData[13]	= (char *)strdup( "" );
		pData[14]	= (char *)strdup( "" );
		pData[15]	= (char *)strdup( "" );
		pData[16]	= (char *)strdup( "" );
		pData[17]	= (char *)strdup( "" );

		pData[18]	= NULL; /* data terminator... see _sqiFreeDataRow */

		lstAppend( hStmt->hTableResult->hRows, pData );

		lstNext( hColHdrs );
	}
	lstClose( hColHdrs );

	hStmt->nRowsAffected	= hStmt->hTableResult->hRows->nItems;

	/* we should be EOL for call to fetch */
	lstLast( hStmt->hTableResult->hRows );
	lstNext( hStmt->hTableResult->hRows );

	return true;
}

==> unixODBC-1.8.8.tar.gz: unixODBC-1.8.8/sqi/sqiColumns.c <==
#include "sqi.h"

int sqiColumns( HSQISTMT hStmt, char *pszTableName )
{
	HSQI		hSQI;
	HLST		hColHdrs;
	char		szBuffer[101];
	char		**pData;
	HSQICOLUMN	pColumn;
    FILE    	*hFile;
	char		szTableFile[ODBC_FILENAME_MAX+1];

	if ( !hStmt )
		return false;
	
	if ( !pszTableName )
		return false;

	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	hSQI	= (HSQI)hStmt->hSQI;

	if ( !_sqiGetTableFileName( hStmt, hSQI->pszCatalogFile, pszTableName, szTableFile ) )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, "Could not get table file name" );
		return false;
	}

	hFile = fopen( szTableFile, "r" );
	if ( !hFile )
		hFile = fopen( szTableFile, "w" );

	if ( !hFile )
	{
		sprintf( hStmt->szMsg, "Could not open table file (%s)", szTableFile );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}

	hColHdrs = lstOpen();
	lstSetFreeFunc( hColHdrs, _sqiFreeColumnHeader );
	
	if ( !_sqiLoadTableColHdrs( hSQI->pszCatalogFile, pszTableName, hColHdrs, hFile ) )
	{
		sprintf( hStmt->szMsg, "Error while loading column headers from (%s)", szTableFile );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		lstClose( hColHdrs );
		fclose( hFile );
		return false;
	}

	fclose( hFile );

	if ( hColHdrs->nItems < 1 )
	{
		sprintf( hStmt->szMsg, "No column headers in (%s)", szTableFile );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		lstClose( hColHdrs );
		return false;
	}


	hStmt->hTableResult 				= malloc( sizeof(SQITABLE) );

	hStmt->hTableResult->pszTableName	= (char*)strdup( "sqiColumns" );
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->nRefs			= 0;
	hStmt->hTableResult->hBase			= NULL;

	/* table header */
	hStmt->hTableResult->hColHdrs	= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hColHdrs, _sqiFreeColumnHeader );

	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 0, "system", "sqiColumns", "TABLE_CAT" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 1, "system", "sqiColumns", "TABLE_SCHEM" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 2, "system", "sqiColumns", "TABLE_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 3, "system", "sqiColumns", "COLUMN_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 4, "system", "sqiColumns", "DATA_TYPE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 5, "system", "sqiColumns", "TYPE_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 6, "system", "sqiColumns", "COLUMN_SIZE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 7, "system", "sqiColumns", "BUFFER_LENGTH" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 8, "system", "sqiColumns", "NUM_PREC_RADIX" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 9, "system", "sqiColumns", "DECIMAL_DIGITS" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 10, "system", "sqiColumns", "NULLABLE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 11, "system", "sqiColumns", "REMARKS" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 12, "system", "sqiColumns", "COLUMN_DEF" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 13, "system", "sqiColumns", "SQL_DATA_TYPE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 14, "system", "sqiColumns", "SQL_DATETIME_SUB" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 15, "system", "sqiColumns", "CHAR_OCTET_LENGTH" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 16, "system", "sqiColumns", "ORDINAL_POSITION" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 17, "system", "sqiColumns", "IS_NULLABLE" ) );

	/* table rows (based upon root table col hdrs) */
	hStmt->hTableResult->hRows		= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hRows, _sqiFreeDataRow );

	lstFirst( hColHdrs );
	while ( !lstEOL( hColHdrs ) )
	{
		pData		= malloc( sizeof(char*) * 19 );
		pColumn 	= (HSQICOLUMN)lstGet( hColHdrs );
		pData[0]	= (char *)strdup( pColumn->pszSQL_DESC_CATALOG_NAME );
		pData[1]	= (char *)strdup( pColumn->pszSQL_DESC_SCHEMA_NAME );
		pData[2]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_TABLE_NAME );
		pData[3]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_COLUMN_NAME );
		pData[4]	= (char *)strdup( "" );
		pData[5]	= (char *)strdup( pColumn->pszSQL_DESC_TYPE_NAME );
		sprintf( szBuffer, "%d", pColumn->nSQL_DESC_LENGTH );
		pData[6]	= (char *)strdup( szBuffer );
		pData[7]	= (char *)strdup( "" );
		pData[8]	= (char *)strdup( "" );
		pData[9]	= (char *)strdup( "" );
		pData[10]	= (char *)strdup( (pColumn->nSQL_DESC_NULLABLE ? "0" : "1") );
		pData[11]	= (char *)strdup( "" );
		pData[12]	= (char *)strdup( "" );
		pData[13]	= (char *)strdup( "" );
		pData[14]	= (char *)strdup( "" );
		pData[15]	= (char *)strdup( "" );
		pData[16]	= (char *)strdup( "" );
		pData[17]	= (char *)strdup( "" );

		pData[18]	= NULL; /* data terminator... see _sqiFreeDataRow */

		lstAppend( hStmt->hTableResult->hRows, pData );

		lstNext( hColHdrs );
	}
	lstClose( hColHdrs );

	hStmt->nRowsAffected	= hStmt->hTableResult->hRows->nItems;

	/* we should be EOL for call to fetch */
	lstLast( hStmt->hTableResult->hRows );
	lstNext( hStmt->hTableResult->hRows );

	return true;
}
